/*
Kevin Gagnon - December 27, 2020
Will turn on a solid state relay until set temperature is reached.
<<<if for any reason the lamp stays on forever, there may be risk of fire, consider yourself warned!>>>
todo: add a second temperature probe. The ideal would be to have one in water and one in the air.
*/

//LCD 16x2 over I2C
#include<LiquidCrystal_I2C_Hangul.h>
#include<Wire.h>

//DS18B20 temperature sensor
#include <OneWire.h> 
#include <DallasTemperature.h>
#define ONE_WIRE_BUS 2 
OneWire oneWire(ONE_WIRE_BUS);
DallasTemperature sensors(&oneWire);

LiquidCrystal_I2C_Hangul lcd(0x3F,16,2); //LCD init over I2C
const int ssr = 5; //solid state relay

float targetTemp = 37.0;

unsigned long startTime=0;
unsigned long maxTimeOn = 60000; //in milliseconds
unsigned long breakTime = 10000; //pause time to prevent lamp overheating
int onBreak = false; //true when pausing

void setup() {
  // initialize digital pin LED_BUILTIN as an output.
  String strTarget;
  strTarget = "Target: ";   strTarget += targetTemp;   strTarget += (char)223;   strTarget += "C"; //I'm not good with concatenation maybe ;-)
  
  pinMode(ssr, OUTPUT); //solid state relay output pin
  Serial.begin(115200);
  lcd.init();
  lcd.backlight();
  lcd.print(strTarget);
  sensors.begin(); //init temperature sensor
  digitalWrite(ssr, LOW); //make sure ssr is deactivated

}

// the loop function runs over and over again forever
void loop() {
  String strActual = "Actual: ";
  sensors.requestTemperatures();
  float temp = sensors.getTempCByIndex(0);
  strActual += temp;   strActual += (char)223;  strActual +="C";  //output string for LCD
  
  lcd.setCursor(0,1); //position to second row
  lcd.print(strActual); 

  if(temp>-40) { //temp will be -127 if the sensor is disconnected 
    if(digitalRead(ssr) == LOW) {
      if(!onBreak) {
        //Serial.println("not on break!");
        if(temp<targetTemp){
          digitalWrite(ssr, HIGH);   // turn the LED on (HIGH is the voltage level)
          startTime = millis();
          Serial.println("starting lamp...");
        }
        else {
          Serial.println("stoping lamp...");
          digitalWrite(ssr, LOW);    // turn the LED off by making the voltage LOW
        }
      }
      else { //is the break over?
        if((millis()-startTime)> breakTime) {
          onBreak=false;  
          Serial.println("break is over!");
        }
      }
    }
    else { //light is already on
      if(temp >= targetTemp) {
        Serial.println("stoping lamp...");
        digitalWrite(ssr, LOW); 
      }
      if((millis()-startTime) > maxTimeOn) {
        //Serial.print("elaspsed:");
        //Serial.println((millis()-startTime));
        onBreak=true;
        digitalWrite(ssr, LOW);
        startTime=millis();
        Serial.println("starting break");
      }
    }
  }
  else  {
      digitalWrite(ssr, LOW);
      Serial.println("sensor is probably disconnected");
  }
    
  delay(1000);                       // wait for a second
    
//  Serial.println(temp); 
  Serial.println(millis()-startTime);
}
